import pymc3
from tvatoj.helpers.helpers import info
from tvatoj.models.tva_base_model import TVABaseModel
from theano.tensor import mean

class TVANcHierachicalSingleRateModelConditions(TVABaseModel):
    
    def vp_rate(self):
        with self.model:
            # Probe rate priors and transformation
            self.vp_mean = pymc3.Uniform('vp_mean',0, 0.200, shape=(self.ncnd))
            self.vp_sd = pymc3.HalfCauchy('vp_sd', beta = 0.01, shape=(self.ncnd))
            self.etap =  pymc3.Normal('etap', mu = 0, sd = 1, shape=(self.nidx,self.ncnd) )
            temp = self.vp_mean + self.etap * self.vp_sd
            self.vp = pymc3.Deterministic('vp', temp)
            
        
    def vr_rate(self):
        with self.model:
          info("Setting reference rate to equal probe rate")
          self.vr = self.vp

    def derived_estimates(self):
        if self.ncnd == 2:
            info("Detected two conditions. Calculating v_diff between them!")
            v_diff = pymc3.Deterministic('v_diff', self.vp[:,0] - self.vp[:,1])
            v_diff_mean = pymc3.Deterministic('v_diff_mean', mean(v_diff))
	
    
   
        
    
    
