import pymc3
from tvatoj.models.tva_base_model import TVABaseModel
from tvatoj.helpers.helpers import info
from theano.tensor import mean

class TVANcHierarchicalModel(TVABaseModel):
    
    def vp_rate(self):
        with self.model:
            info("Probe Rate setup in TVANcHierachicalModel with DUPPL = " + 
                 str(self.default_uniform_probe_prior_limit))
            ## Probe rate priors and transformation
            self.vp_mean = pymc3.Uniform('vp_mean',0, self.default_uniform_probe_prior_limit, shape=(self.ncnd))
            vp_sd = pymc3.HalfCauchy('vp_sd', beta = 0.01, shape=(self.ncnd))
            etap =  pymc3.Normal('etap', mu = 0, sd = 1, shape=(self.nidx,self.ncnd) )
            self.vp = pymc3.Deterministic('vp', self.vp_mean + etap * vp_sd)
            
    def vr_rate(self):
        with self.model:
            info("Reference Rate setup in TVANcHierachicalModel with DURPL = " + 
                 str(self.default_uniform_reference_prior_limit))
            self.vr_mean = pymc3.Uniform('vr_mean',0.0, self.default_uniform_reference_prior_limit, shape=(self.ncnd))  
            vr_sd = pymc3.HalfCauchy('vr_sd', beta = 0.01,  shape=(self.ncnd))
            etar =  pymc3.Normal('etar', mu = 0, sd = 1, shape=(self.nidx,self.ncnd) )
            self.vr = pymc3.Deterministic('vr', self.vr_mean + etar * vr_sd)
            
    def derived_estimates(self):
        super().derived_estimates()
        #TODO: AK check whether I handled dimensions correctly and mean is for conditions
        self.C_mean = pymc3.Deterministic('C_mean', mean(self.C, axis=0))
        self.w_p_mean = pymc3.Deterministic('w_p_mean', self.vp_mean / (self.vr_mean + self.vp_mean) )