'''
Created on Jul 31, 2016#

@author: jeti
'''

import pymc3
from numpy import min, max, linspace, arange, sqrt
from scipy.stats import kde
from matplotlib.pylab import plt
from numpy import argsort



def plot_hdi_bracket(hdi=[0.025,0.040],modev=0.030,bracket_height=10, 
                    fane_height=150, size_hint=0.001, top_padd=0.02, 
                    fane_ext=0.02,  ax=None, name='', unit='', 
                    text_x_offset=0):
    '''
    Plot a curly HDI bracket. 
    :param hdi: Lower and upper HDI boundary as [lower, upper].
    :param modev: Value of the mode.
    :param bracket_height: Height of the bracket.
    :param fane_height: Height of the vertical line extending from the bracket
    :param size_hint: To be used for scaling the curls. It's not quite intuitive
                     but as a reference: 0.001 looks good for rates with typical
                     posteriors.
    :param top_padd: Padding at the top to avoid the text leaving the plot.
    :param fance_ext: TODO: is this still useful?
    :param ax: Axes object for plotting. If it is 'None', a new plot is created.
    :param name: Name of the rate parameter. Can use LaTex.
    :param unit: String which is added to indicate the unit. Usually: 'Hz'.
    :param text_x_offset: For adjusting the text position in the x direction.
    '''
    if ax is None:
        ax = plt.subplot()
    y = []
    hdi_range = arange(hdi[0],hdi[1],(hdi[1]-hdi[0])/500.0)
    for h in hdi_range:
        
        if h-hdi[0] < size_hint * 1000:
            y.append(bracket_height * 
                     sqrt(float(h-hdi[0]))/sqrt(size_hint*1000))
        elif hdi[1]-h < size_hint*1000:
            y.append(bracket_height * 
                     sqrt(float(hdi[1]-h))/sqrt(size_hint*1000))
        elif abs(modev-h) < size_hint*1000:
            y.append(bracket_height+(fane_height/10.0)*
                     (1-sqrt((abs(modev-h)))/sqrt(size_hint*1000)))
        else:
            y.append(bracket_height)

    ax.plot((modev, modev), (bracket_height+(fane_height/10.0), 
                             fane_height), 'k-')
    ax.plot((modev, modev), (fane_height, fane_height+fane_ext), 'k:')
    
    ax.plot((0,0), (fane_height,fane_height+fane_ext+(size_hint*10)+top_padd),
            linestyle='dotted', color='k',alpha=0.0)
    ax.plot(hdi_range,y,color='black')

    ax.text(modev+(size_hint*10)+text_x_offset, fane_height+fane_ext+
            (size_hint*3),name +" = " + "%2.2f"%(modev) + " " + 
            unit ,{'size':14},clip_on=True)
        

def plot_rate_posterior(trace_values,ax=None,kde_plot=True,factor=1, **kwargs):
        trace_values=trace_values*factor
        mode=0
        height=0
        if ax is None:
            ax = plt.subplot()
        if kde_plot:
            density = kde.gaussian_kde(trace_values)
           
            mode = trace_values[argsort(density(trace_values))[-1]]
            height = density(mode)
            l = min(trace_values)
            u = max(trace_values)
            x = linspace(0, 1, 100) * (u - l) + l
            ax.plot(x, density(x), 'k') #,**kwargs)
            ax.fill_between(x, 0, density(x), alpha=0.5, **kwargs)
            print(mode)
        else:
            print('Not implemented yet!')
        return ax, mode, height

def plot_rates(trace, rates=['vp:mean','vr:mean'], names=[r'$v_p$',r'$v_p$'],
               factors=[1000,1000], units=['Hz','Hz'], ext=[0,0], top_padd=0,
               size_hint=0.001, kde_plot=True, ax=None, text_x_offset=0,
               **kwargs):
    ax=ax
  
    if ax is None:
            ax = plt.subplot()
            
    for i,rate in enumerate(rates):
        x = trace.get_values(rate)
        ax, modev, height = plot_rate_posterior(x, ax=ax, kde_plot=True, 
                                                factor=factors[i], **kwargs)
        hdi = pymc3.hpd(trace.get_values(rate)*factors[i])
        plot_hdi_bracket(hdi, modev, fane_height=height, 
                         bracket_height=(height/8.0)*(i+1), fane_ext=ext[i], 
                         name=names[i], unit=units[i], ax=ax, top_padd=top_padd,
                         size_hint=size_hint, text_x_offset=text_x_offset)
        
    return ax

