from matplotlib.pylab import plt
from matplotlib.pyplot import xlabel
import numpy as np

def plot_toj(toj, ax=None, relative = True, vline=True, hline=True, **kwargs):
    '''
    Plot TOJ raw data.
    :param toj: is the TOJ DataFrame
    :param relative: Whether to plot relative frequency (set to False for absolute counts)
    '''
    if ax is None:
        ax = plt.subplot()
    
    
    kwargs.setdefault('marker', 'x')
    kwargs.setdefault('linestyle', '')
    if relative:
        #toj['Rel'] = toj['Count']/toj['Repetitions']
        ax = toj.plot(x='SOA',y='Relative',ax=ax,**kwargs)
        ax.set_xlabel("SOA [ms]")
        ax.set_ylim(0,1)
        if hline:
            ax.axhline(y=0.5, color='k', alpha=0.6, linestyle=':')
        
    else:
        ax = toj.plot(kwargs,x='SOA',y='Counts',ax=ax)
    ax.set_xlabel('SOA [ms]')
    
    if vline:    
        ax.axvline(x=0,color='k', alpha=0.6, linestyle=':')

    return ax


def plot_tojs(tojs, condition_names, relative=True, vline=True, hline=True, marker='+', cmap='viridis', kind='scatter', **kwargs):
    num_subjects = tojs['Running_subject_number'].nunique()

    cmap = plt.get_cmap(cmap)  # color map, see http://stackoverflow.com/questions/31883097/elegant-way-to-match-a-string-to-a-random-color-matplotlib
    colors = cmap(np.linspace(0, 1, len(condition_names)))

    fig, axes = plt.subplots(nrows=num_subjects, ncols=1, **kwargs)
    for i in range(num_subjects):
        subject_tojs = tojs.loc[tojs['Running_subject_number'] == i]

        for condition_id, condition_name in enumerate(condition_names):
            color = colors[condition_id]
            if relative:
                ax = subject_tojs.loc[subject_tojs['Condition'] == condition_name].plot(
                    kind=kind,
                    x='SOA', y='Relative',
                    ax=axes[i], label=condition_name, marker=marker, c=color, **kwargs
                )
            else:
                ax = subject_tojs.loc[subject_tojs['Condition'] == condition_name].plot(
                    kind=kind,
                    x='SOA', y='Counts',
                    ax=axes[i], label=condition_name, marker=marker, c=color, **kwargs
                )

        if relative:
            ax.set_ylim(0, 1)
            if hline:
                ax.axhline(y=0.5, color='k', alpha=0.6, linestyle=':')

        axes[i].set_xlabel('SOA [ms]')
        axes[i].set_title('Subject running no. {0}'.format(i))

        if vline:
            axes[i].axvline(x=0, color='k', alpha=0.6, linestyle=':')
            
